@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 2 -*- */
+/* -*- Mode: C; c-file-style: "gnu"; tab-width: 8 -*- */
 /* Copyright (C) 2004 Carlos Garnacho
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,74 +21,21 @@
 #include <glib/gi18n.h>
 
 #include "gst.h"
-#include "gst-network-tool.h"
-#include "network-iface.h"
+#include "network-tool.h"
 #include "ifaces-list.h"
 #include "callbacks.h"
+#include "hosts.h"
+#include "nm-integration.h"
 
 extern GstTool *tool;
 
-static void
-enable_iface (GstIface *iface)
-{
-  GtkWidget *dialog;
-  gboolean   success;
-
-  success = gst_iface_enable (iface);
-  gst_iface_set_auto (iface, success);
-
-  if (!success)
-    {
-      dialog = gtk_message_dialog_new (GTK_WINDOW (tool->main_dialog),
-				       GTK_DIALOG_MODAL,
-				       GTK_MESSAGE_ERROR,
-				       GTK_BUTTONS_CLOSE,
-				       _("Could not enable the interface %s"),
-				       gst_iface_get_dev (iface),
-				       NULL);
-      gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						_("Check that the settings are correct for "
-						  "this network and that the computer is correctly "
-						  "connected to it."),
-						NULL);
-      gtk_dialog_run (GTK_DIALOG (dialog));
-      gtk_widget_destroy (dialog);
-    }
-}
-
 void
 on_table_selection_changed (GtkTreeSelection *selection, gpointer data)
 {
-  GtkWidget    *properties, *activate, *deactivate;
-  GtkTreeModel *model;
-  GtkTreeIter   iter;
-  GstIface     *iface;
-  gboolean      enabled, configured;
-  gboolean      state_properties, state_activate, state_deactivate;
+  GtkWidget    *properties;
 
   properties = gst_dialog_get_widget (tool->main_dialog, "properties_button");
-  activate   = gst_dialog_get_widget (tool->main_dialog, "activate_button");
-  deactivate = gst_dialog_get_widget (tool->main_dialog, "deactivate_button");
-
-  state_properties = state_activate = state_deactivate = FALSE;
-
-  if (gtk_tree_selection_get_selected (selection, &model, &iter))
-    {
-      gtk_tree_model_get (model, &iter,
-			  COL_OBJECT, &iface,
-			  COL_CONFIGURED, &configured,
-			  -1);
-      enabled = gst_iface_get_enabled (iface);
-      g_object_unref (iface);
-
-      state_properties = TRUE;
-      state_activate = configured && !enabled;
-      state_deactivate = configured && enabled;
-    }
-
-  gtk_widget_set_sensitive (properties, state_properties);
-  gtk_widget_set_sensitive (activate,   state_activate);
-  gtk_widget_set_sensitive (deactivate, state_deactivate);
+  gtk_widget_set_sensitive (properties, gtk_tree_selection_get_selected_rows (selection, NULL) != 0);
 }
 
 void
@@ -97,8 +44,8 @@ on_iface_properties_clicked (GtkWidget *widget, gpointer data)
   GstConnectionDialog *dialog;
   GtkTreeSelection *selection;
   GtkTreeModel *model;
-  GtkTreeIter   iter;
-  GstIface *iface;
+  GtkTreeIter iter;
+  OobsIface *iface;
 
   selection = gtk_tree_view_get_selection (GST_NETWORK_TOOL (tool)->interfaces_list);
 
@@ -110,8 +57,6 @@ on_iface_properties_clicked (GtkWidget *widget, gpointer data)
 
       dialog = GST_NETWORK_TOOL (tool)->dialog;
       connection_dialog_prepare (dialog, iface);
-      g_object_unref (iface);
-
       gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog), GTK_WINDOW (tool->main_dialog));
       gtk_widget_show (dialog->dialog);
     }
@@ -121,12 +66,37 @@ void
 on_iface_active_changed (GtkWidget *widget, gpointer data)
 {
   GstConnectionDialog *dialog;
+  GtkWidget *roaming_active;
   gboolean active;
 
   dialog = GST_NETWORK_TOOL (tool)->dialog;
   active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 
   connection_dialog_set_sensitive (dialog, active);
+
+  roaming_active = gst_dialog_get_widget (tool->main_dialog, "connection_device_roaming");
+  g_signal_handlers_block_by_func (roaming_active, on_iface_roaming_changed, data);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (roaming_active), !active);
+  g_signal_handlers_unblock_by_func (roaming_active, on_iface_roaming_changed, data);
+}
+
+void
+on_iface_roaming_changed (GtkWidget *widget, gpointer data)
+{
+  GstConnectionDialog *dialog;
+  GtkWidget *device_active;
+  gboolean active;
+
+  dialog = GST_NETWORK_TOOL (tool)->dialog;
+  active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
+
+  /* roaming changed and device active have just inverted logics */
+  connection_dialog_set_sensitive (dialog, !active);
+
+  device_active = gst_dialog_get_widget (tool->main_dialog, "connection_device_active");
+  g_signal_handlers_block_by_func (device_active, on_iface_active_changed, data);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (device_active), !active);
+  g_signal_handlers_unblock_by_func (device_active, on_iface_active_changed, data);
 }
 
 void
@@ -138,66 +108,100 @@ on_bootproto_changed (GtkWidget *widget, gpointer data)
 
   dialog = GST_NETWORK_TOOL (tool)->dialog;
   pos = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
-  enabled = (pos == 0);
+  enabled = (pos == 1);
 
   gtk_widget_set_sensitive (dialog->address, enabled);
   gtk_widget_set_sensitive (dialog->netmask, enabled);
   gtk_widget_set_sensitive (dialog->gateway, enabled);
 }
 
-void
-on_connection_cancel_clicked (GtkWidget *widget, gpointer data)
+static void
+cancel_connection_dialog (GstTool *tool)
 {
   GstConnectionDialog *dialog;
 
   dialog = GST_NETWORK_TOOL (tool)->dialog;
   g_object_unref (dialog->iface);
-  gtk_widget_hide (dialog->dialog);
+  connection_dialog_hide (dialog);
 
   if (dialog->standalone)
     gtk_main_quit ();
 }
 
-void
-on_connection_ok_clicked (GtkWidget *widget, gpointer data)
+static gboolean
+toggle_nm (gpointer data)
+{
+  GstNetworkTool *tool = GST_NETWORK_TOOL (data);
+  NMState state;
+
+  state = nm_integration_get_state (tool);
+
+  if (state == NM_STATE_DISCONNECTED ||
+      state == NM_STATE_CONNECTING ||
+      state == NM_STATE_CONNECTED)
+    {
+      /* "reboot" NM */
+      nm_integration_sleep (tool);
+      nm_integration_wake (tool);
+    }
+
+  return FALSE;
+}
+
+static void
+on_configuration_changed (OobsObject *object,
+			  OobsResult  result,
+			  gpointer    data)
+{
+  /* suckety suck, but NM seems not to realize
+   * immediately that the interface status changed */
+  g_timeout_add (2000, toggle_nm, data);
+}
+
+static void
+accept_connection_dialog (GstTool *tool)
 {
   GstConnectionDialog *dialog;
   GtkTreeSelection    *selection;
-  GtkTreeModel        *model;
   GtkTreeIter          iter;
-  GstIface            *iface;
 
   dialog = GST_NETWORK_TOOL (tool)->dialog;
-  gtk_widget_hide (dialog->dialog);
+  connection_dialog_hide (dialog);
 
   if (dialog->changed)
     {
       connection_save (dialog);
       selection = gtk_tree_view_get_selection (GST_NETWORK_TOOL (tool)->interfaces_list);
 
-      if (gtk_tree_selection_get_selected (selection, &model, &iter))
+      if (gtk_tree_selection_get_selected (selection, NULL, &iter))
         {
-	  gtk_tree_model_get (model, &iter, COL_OBJECT, &iface, -1);
-
-	  /* modify the config realtime */
-	  if (gst_iface_get_enabled (iface))
-	    enable_iface (iface);
-
 	  ifaces_model_modify_interface_at_iter (&iter);
-	  g_object_unref (iface);
-
 	  g_signal_emit_by_name (G_OBJECT (selection), "changed");
-	  gst_dialog_modify (tool->main_dialog);
 	}
+
+      gst_tool_commit_async (tool, OOBS_OBJECT (GST_NETWORK_TOOL (tool)->ifaces_config),
+			     _("Changing interface configuration"),
+			     on_configuration_changed, tool);
     }
 
   g_object_unref (dialog->iface);
 
   if (dialog->standalone)
-    {
-      gtk_signal_emit_by_name (GTK_OBJECT (tool->main_dialog), "apply", tool);
-      gtk_main_quit ();
-    }
+    gtk_main_quit ();
+}
+
+void
+on_connection_response (GtkWidget *widget,
+			gint       response,
+			gpointer   data)
+{
+  GstDialog *dialog = GST_DIALOG (data);
+  GstTool *tool = gst_dialog_get_tool (dialog);
+
+  if (response == GTK_RESPONSE_OK)
+    accept_connection_dialog (tool);
+  else
+    cancel_connection_dialog (tool);
 }
 
 void
@@ -322,55 +326,6 @@ on_table_popup_menu (GtkWidget *widget, gpointer data)
   return TRUE;
 }
 
-static void
-enable_disable_iface (GstNetworkTool *network_tool, gboolean enable)
-{
-  GtkTreeSelection *selection;
-  GtkTreeModel *model;
-  GtkTreeIter   iter;
-  GstIface     *iface;
-
-  selection = gtk_tree_view_get_selection (network_tool->interfaces_list);
-
-  if (gtk_tree_selection_get_selected (selection, &model, &iter))
-    {
-      gtk_tree_model_get (model, &iter,
-			  COL_OBJECT, &iface,
-			  -1);
-      if (enable)
-	enable_iface (iface);
-      else
-        {
-	  gst_iface_disable (iface);
-	  gst_iface_set_auto (iface, FALSE);
-	}
-
-      ifaces_model_modify_interface_at_iter (&iter);
-      g_object_unref (iface);
-
-      /* we need this to update the buttons state */
-      g_signal_emit_by_name (G_OBJECT (selection), "changed");
-
-      /* update gateway settings */
-      if (gtk_combo_box_get_active (network_tool->gateways_list) == -1)
-	gtk_combo_box_set_active (network_tool->gateways_list, 0);
-
-      gst_dialog_modify (tool->main_dialog);
-    }
-}
-
-void
-on_activate_button_clicked (GtkWidget *widget, gpointer data)
-{
-  enable_disable_iface (GST_NETWORK_TOOL (tool), TRUE);
-}
-
-void
-on_deactivate_button_clicked (GtkWidget *widget, gpointer data)
-{
-  enable_disable_iface (GST_NETWORK_TOOL (tool), FALSE);
-}
-
 void
 on_host_aliases_add_clicked (GtkWidget *widget, gpointer data)
 {
@@ -396,17 +351,24 @@ void
 on_host_aliases_delete_clicked (GtkWidget *widget, gpointer data)
 {
   GtkTreeSelection *selection;
-  GtkTreeView      *list;
-  GtkTreeModel     *model;
-  GtkTreeIter       iter;
+  GtkTreeView *list;
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  OobsList *hosts_list;
+  OobsListIter *list_iter;
 
   list  = GST_NETWORK_TOOL (tool)->host_aliases_list;
   selection = gtk_tree_view_get_selection (list);
 
   if (gtk_tree_selection_get_selected (selection, &model, &iter))
     {
+      gtk_tree_model_get (model, &iter, COL_HOST_ITER, &list_iter, -1);
+      hosts_list = oobs_hosts_config_get_static_hosts (GST_NETWORK_TOOL (tool)->hosts_config);
+      oobs_list_remove (hosts_list, list_iter);
       gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
-      gst_dialog_modify (tool->main_dialog);
+
+      oobs_list_iter_free (list_iter);
+      oobs_object_commit (OOBS_OBJECT (GST_NETWORK_TOOL (tool)->hosts_config));
     }
 }
 
@@ -425,60 +387,6 @@ on_dialog_changed (GtkWidget *widget, gpointer data)
   connection_check_fields (network_tool->dialog);
 }
 
-gboolean
-callbacks_check_hostname_hook (GstDialog *dialog, gpointer data)
-{
-  GstNetworkTool *network_tool;
-  gchar          *hostname_old, *hostname_new;
-  xmlNodePtr      root, node;
-  GtkWidget      *d;
-  gint            res;
-
-  network_tool = GST_NETWORK_TOOL (dialog->tool);
-  root = gst_xml_doc_get_root (dialog->tool->config);
-  node = gst_xml_element_find_first (root, "hostname");
-
-  hostname_old = gst_xml_element_get_content (node);
-  hostname_new = (gchar *) gtk_entry_get_text (network_tool->hostname);
-
-  if (hostname_old && hostname_new &&
-      (strcmp (hostname_new, hostname_old) != 0))
-    {
-      d = gtk_message_dialog_new (GTK_WINDOW (tool->main_dialog),
-				  GTK_DIALOG_MODAL,
-				  GTK_MESSAGE_WARNING,
-				  GTK_BUTTONS_NONE,
-				  _("The host name has changed"),
-				  NULL);
-      gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (d),
-						_("This will prevent you "
-						  "from launching new applications, and so you will "
-						  "have to log in again. Continue anyway?"),
-						NULL);
-      gtk_dialog_add_buttons (GTK_DIALOG (d),
-			      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-			      _("Change _Host name"), GTK_RESPONSE_ACCEPT,
-			      NULL);
-
-      res = gtk_dialog_run (GTK_DIALOG (d));
-      gtk_widget_destroy (d);
-
-      switch (res)
-        {
-	case GTK_RESPONSE_ACCEPT:
-	  g_free (hostname_old);
-	  return TRUE;
-	case GTK_RESPONSE_CANCEL:
-	default:
-	  gtk_entry_set_text (network_tool->hostname, hostname_old);
-	  return FALSE;
-	}
-    }
-
-  g_free (hostname_old);
-  return TRUE;
-}
-
 gboolean
 on_ip_address_focus_out (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 {
@@ -489,39 +397,121 @@ on_ip_address_focus_out (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 }
 
 void
-on_gateway_combo_changed (GtkWidget *widget, gpointer data)
+on_iface_toggled (GtkCellRendererToggle *renderer,
+		  gchar                 *path_str,
+		  gpointer               data)
 {
-  GtkTreeModel *model;
-  GtkTreeIter   iter;
-  GstIface     *iface;
-  gchar        *dev, *gateway;
+  GtkTreePath *path;
+  GtkTreeModel *model = GTK_TREE_MODEL (data);
+  GtkTreeIter iter;
+  gboolean active, inconsistent;
+  OobsIface *iface;
 
-  if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (widget), &iter))
+  path = gtk_tree_path_new_from_string (path_str);
+
+  if (gtk_tree_model_get_iter (model, &iter, path))
     {
-      model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
-
       gtk_tree_model_get (model, &iter,
+			  COL_ACTIVE, &active,
+			  COL_INCONSISTENT, &inconsistent,
 			  COL_OBJECT, &iface,
 			  -1);
 
-      /* FIXME: this won't work for DHCP nor PPP ifaces */
-      g_object_get (G_OBJECT (iface),
-		    "iface-dev", &dev,
-		    "iface-gateway", &gateway,
-		    NULL);
+      if (!inconsistent)
+	{
+	  active ^= 1;
 
-      if (dev && gateway)
-	gst_tool_run_set_directive (tool, NULL, NULL,
-				    "set_gateway", dev, gateway, NULL);
+	  oobs_iface_set_active (iface, active);
+	  oobs_iface_set_auto (iface, active);
+	  ifaces_model_modify_interface_at_iter (&iter);
 
-      g_free (dev);
-      g_free (gateway);
+	  gst_tool_commit_async (tool, OOBS_OBJECT (GST_NETWORK_TOOL (tool)->ifaces_config),
+				 _("Changing interface configuration"),
+				 on_configuration_changed, tool);
+	}
+
+      g_object_unref (iface);
     }
+
+  gtk_tree_path_free (path);
+}
+
+void
+on_entry_changed (GtkWidget *widget, gpointer data)
+{
+  g_object_set_data (G_OBJECT (widget), "content-changed", GINT_TO_POINTER (TRUE));
 }
 
 gboolean
-on_connection_dialog_close (GtkWidget *widget, GdkEvent *event, gpointer data)
+on_hostname_focus_out (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 {
-  gtk_widget_hide (widget);
-  return TRUE;
+  GstNetworkTool *tool;
+  gboolean changed;
+  const gchar *hostname;
+
+  tool = GST_NETWORK_TOOL (gst_dialog_get_tool (GST_DIALOG (data)));
+  changed = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "content-changed"));
+  hostname = gtk_entry_get_text (GTK_ENTRY (widget));
+
+  if (changed && hostname && *hostname)
+    {
+      GtkWidget *dialog;
+      gint res;
+
+      dialog = gtk_message_dialog_new (GTK_WINDOW (GST_TOOL (tool)->main_dialog),
+				       GTK_DIALOG_MODAL,
+				       GTK_MESSAGE_WARNING,
+				       GTK_BUTTONS_NONE,
+				       _("The host name has changed"),
+				       NULL);
+      gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						_("This will prevent you "
+						  "from launching new applications, and so you will "
+						  "have to log in again. Continue anyway?"),
+						NULL);
+      gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+			      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			      _("Change _Host name"), GTK_RESPONSE_ACCEPT,
+			      NULL);
+
+      res = gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+
+      if (res == GTK_RESPONSE_ACCEPT)
+	{
+	  oobs_hosts_config_set_hostname (tool->hosts_config, hostname);
+	  oobs_object_commit (OOBS_OBJECT (tool->hosts_config));
+	}
+      else
+	{
+	  gtk_entry_set_text (GTK_ENTRY (widget),
+			      oobs_hosts_config_get_hostname (tool->hosts_config));
+	}
+    }
+
+  g_object_set_data (G_OBJECT (widget), "content-changed", GINT_TO_POINTER (FALSE));
+
+  return FALSE;
+}
+
+gboolean
+on_domain_focus_out (GtkWidget *widget, GdkEventFocus *event, gpointer data)
+{
+  GstNetworkTool *tool;
+  gboolean changed;
+  const gchar *domain;
+
+  tool = GST_NETWORK_TOOL (gst_dialog_get_tool (GST_DIALOG (data)));
+  changed = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "content-changed"));
+  domain = gtk_entry_get_text (GTK_ENTRY (widget));
+
+  if (changed)
+    {
+      oobs_hosts_config_set_domainname (tool->hosts_config, domain);
+      oobs_object_commit (OOBS_OBJECT (tool->hosts_config));
+    }
+
+  g_object_set_data (G_OBJECT (widget), "content-changed", GINT_TO_POINTER (FALSE));
+
+  return FALSE;
 }
