@@ -23,6 +23,8 @@
 #include "gst.h"
 #include "network-tool.h"
 #include "connection.h"
+#include "essid-list.h"
+#include "nm-integration.h"
 
 extern GstTool *tool;
 
@@ -76,18 +78,34 @@ connection_essids_combo_init (GtkComboBoxEntry *combo)
   GtkTreeModel *model;
   GtkCellRenderer *renderer;
 
-  model = GTK_TREE_MODEL (gtk_list_store_new (2, GDK_TYPE_PIXBUF, G_TYPE_STRING));
+  model = GTK_TREE_MODEL (gtk_list_store_new (3, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_INT));
   gtk_combo_box_set_model (GTK_COMBO_BOX (combo), model);
   g_object_unref (model);
 
-  gtk_combo_box_entry_set_text_column (combo, 1);
+  gtk_cell_layout_clear (GTK_CELL_LAYOUT (combo));
 
-  /* Add the pixbuf cell renderer */
+  /* add "crypted" renderer */
   renderer = gtk_cell_renderer_pixbuf_new ();
   gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo),
 			      renderer, FALSE);
   gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (combo),
 				 renderer, "pixbuf", 0);
+
+  /* add "essid" renderer */
+  renderer = gtk_cell_renderer_text_new ();
+  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo),
+			      renderer, TRUE);
+  gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (combo),
+				 renderer, "text", 1);
+
+  /* add "quality" renderer */
+  renderer = gtk_cell_renderer_progress_new ();
+  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo),
+			      renderer, FALSE);
+  gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (combo),
+				 renderer, "value", 2);
+
+  gtk_combo_box_entry_set_text_column (combo, 1);
 }
 
 static void
@@ -143,60 +161,53 @@ ethernet_dialog_check_fields (GstConnectionDialog *dialog)
 	   (!gateway || gst_filter_check_ip_address (gateway) == GST_ADDRESS_IPV4)));
 }
 
+#ifdef HAVE_LIBIW_H
+/* perhaps there should be a GstEssidListModel class to hide this
+ * stuff, but I'll leave that as a code beautification exercise */
 static void
-wireless_essid_populate_model (GtkComboBox *combo, const gchar *dev)
+on_essid_list_changed (GstEssidList        *list,
+		       GstConnectionDialog *dialog)
 {
+  GList *elem;
+  GstEssidListEntry *entry;
   GtkTreeModel *model;
-  GtkTreeIter   iter;
-  xmlNodePtr    root, node;
-  xmlDoc       *doc;
-  GdkPixbuf    *locked, *unlocked, *pix;
-  gchar        *essid;
-  gboolean      encrypted;
+  GtkTreeIter iter;
+  GdkPixbuf *locked, *unlocked, *pixbuf;
 
-  /* FIXME
-  model = gtk_combo_box_get_model (combo);
+  elem = gst_essid_list_get_list (dialog->essid_list);
+  model = gtk_combo_box_get_model (GTK_COMBO_BOX (dialog->essid));
   gtk_list_store_clear (GTK_LIST_STORE (model));
 
-  doc   = gst_tool_run_get_directive (tool, NULL, "detect_essids", dev, NULL);
-  g_return_if_fail (doc != NULL);
+  locked = gtk_icon_theme_load_icon (tool->icon_theme, "gnome-dev-wavelan-encrypted", 16, 0, NULL);
+  unlocked = gtk_icon_theme_load_icon (tool->icon_theme, "gnome-dev-wavelan", 16, 0, NULL);
 
-  locked = gtk_icon_theme_load_icon (GST_NETWORK_TOOL (tool)->icon_theme,
-				     "gnome-dev-wavelan-encrypted", 16, 0, NULL);
-  unlocked = gtk_icon_theme_load_icon (GST_NETWORK_TOOL (tool)->icon_theme,
-				       "gnome-dev-wavelan", 16, 0, NULL);
-  root  = gst_xml_doc_get_root (doc);
-
-  for (node = gst_xml_element_find_first (root, "network");
-       node; node = gst_xml_element_find_next (node, "network"))
+  while (elem)
     {
-      essid = gst_xml_get_child_content (node, "essid");
-      encrypted = gst_xml_element_get_boolean (node, "encrypted");
-
-      pix = (encrypted) ?
-	g_object_ref (locked) :
-	g_object_ref (unlocked);
+      entry = elem->data;
+      pixbuf = (entry->encrypted) ? locked : unlocked;
 
       gtk_list_store_append (GTK_LIST_STORE (model), &iter);
       gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-			  0, pix, 1, essid, -1);
-
-      g_object_unref (pix);
-      g_free (essid);
+			  0, pixbuf,
+			  1, entry->essid,
+			  2, (gint) (entry->quality * 100),
+			  -1);
+      elem = elem->next;
     }
 
   g_object_unref (locked);
   g_object_unref (unlocked);
-  */
 }
+#endif
 
 static void
 wireless_dialog_prepare (GstConnectionDialog *dialog)
 {
-  gchar *essid, *key;
+  gchar *essid, *key, *dev;
   OobsWirelessKeyType key_type;
 
   g_object_get (G_OBJECT (dialog->iface),
+		"device", &dev,
 		"essid", &essid,
 		"key", &key,
 		"key-type", &key_type,
@@ -206,12 +217,16 @@ wireless_dialog_prepare (GstConnectionDialog *dialog)
   gtk_entry_set_text (GTK_ENTRY (GTK_BIN (dialog->essid)->child), (essid) ? essid : "");
   gtk_entry_set_text (GTK_ENTRY (dialog->wep_key), (key) ? key : "");
 
-  /* FIXME
-  wireless_essid_populate_model (GTK_COMBO_BOX (dialog->essid), dev);
-  */
+#ifdef HAVE_LIBIW_H
+  dialog->essid_list = gst_essid_list_new (dev);
+  on_essid_list_changed (dialog->essid_list, dialog);
+  g_signal_connect (G_OBJECT (dialog->essid_list), "changed",
+		    G_CALLBACK (on_essid_list_changed), dialog);
+#endif
 
   g_free (essid);
   g_free (key);
+  g_free (dev);
 }
 
 static void
@@ -382,6 +397,7 @@ connection_dialog_init (GstTool *tool)
   gcd->options_page = gst_dialog_get_widget (tool->main_dialog, "connection_options_page");
 
   gcd->connection_configured = gst_dialog_get_widget (tool->main_dialog, "connection_device_active");
+  gcd->roaming_configured = gst_dialog_get_widget (tool->main_dialog, "connection_device_roaming");
 
   /* ethernet */
   gcd->bootproto_combo = gst_dialog_get_widget (tool->main_dialog, "connection_bootproto");
@@ -480,6 +496,26 @@ prepare_dialog_layout (GstConnectionDialog *dialog,
     }
 }
 
+void
+general_prepare (GstConnectionDialog *dialog,
+		 OobsIface           *iface)
+{
+  NMState state;
+
+  state = nm_integration_get_state (GST_NETWORK_TOOL (tool));
+
+  if (nm_integration_iface_supported (iface) && state != NM_STATE_UNKNOWN)
+    {
+      gtk_widget_show (dialog->roaming_configured);
+      gtk_widget_hide (dialog->connection_configured);
+    }
+  else
+    {
+      gtk_widget_hide (dialog->roaming_configured);
+      gtk_widget_show (dialog->connection_configured);
+    }
+}
+
 void
 connection_dialog_prepare (GstConnectionDialog *dialog, OobsIface *iface)
 {
@@ -497,8 +533,11 @@ connection_dialog_prepare (GstConnectionDialog *dialog, OobsIface *iface)
   prepare_dialog_layout (dialog, iface);
   connection_dialog_set_sensitive (dialog, active);
 
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->roaming_configured), !active);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->connection_configured), active);
 
+  general_prepare (dialog, iface);
+
   if (OOBS_IS_IFACE_ISDN (iface) || OOBS_IS_IFACE_MODEM (iface))
     {
       isdn_dialog_prepare (dialog);
@@ -575,20 +614,7 @@ connection_check_fields (GstConnectionDialog *dialog)
 gchar*
 connection_detect_modem (void)
 {
-  xmlNodePtr  root;
-  xmlDoc     *doc;
-  gchar      *device;
-
-  /* FIXME
-  doc= gst_tool_run_get_directive (tool, NULL, "detect_modem", NULL);
-  g_return_val_if_fail (doc != NULL, NULL);
-
-  root   = gst_xml_doc_get_root (doc);
-  device = gst_xml_get_child_content (root, "device");
-  xmlFreeDoc (doc);
-	
-  return device;
-  */
+  /* FIXME: Detect modem */
   return NULL;
 }
 
@@ -623,3 +649,18 @@ connection_dialog_set_sensitive (GstConnectionDialog *dialog, gboolean active)
   gtk_widget_set_sensitive (dialog->isp_frame,      active);
   gtk_widget_set_sensitive (dialog->account_frame,  active);
 }
+
+void
+connection_dialog_hide (GstConnectionDialog *dialog)
+{
+  gtk_widget_hide (dialog->dialog);
+
+#ifdef HAVE_LIBIW_H
+  /* get rid of the essid list, if any */
+  if (dialog->essid_list)
+    {
+      g_object_unref (dialog->essid_list);
+      dialog->essid_list = NULL;
+    }
+#endif
+}
